@@ -1,6 +1,6 @@
 /***
 |''Name''|TiddlySpaceFollowingPlugin|
-|''Version''|0.6.13|
+|''Version''|0.6.15|
 |''Description''|Provides a following macro|
 |''Author''|Jon Robson|
 |''Requires''|TiddlySpaceConfig TiddlySpaceTiddlerIconsPlugin|
@@ -48,12 +48,13 @@ If no name is given eg. {{{<<following>>}}} or {{{<<follow>>}}} it will default
 
 var tweb = config.extensions.tiddlyweb;
 var tiddlyspace = config.extensions.tiddlyspace;
+var currentSpace = tiddlyspace.currentSpace.name;
 
 var shadows = config.shadowTiddlers;
-config.annotations.ScanTemplate = "This tiddler is a template used in the display of tiddlers from spaces you are following. Use the wildcard {{{$1}}} to access the spaceField. To access attributes use the view macro e.g. {{{<<view title text>>}}}";
+config.annotations.ScanTemplate = "This tiddler is the default template used in the display of tiddlers founding using the tsScan macro. To access attributes use the view macro e.g. {{{<<view title text>>}}}";
 shadows.ScanTemplate = "<<view modifier SiteIcon width:24 height:24 spaceLink:yes label:no>> <<view title link>>";
-shadows.FollowersTemplate = "<<view server.bag SiteIcon width:24 height:24 spaceLink:yes label:no>> @$1";
-shadows.FollowingTemplate = "<<view server.bag SiteIcon width:24 height:24 spaceLink:yes label:no>> @$1";
+shadows.FollowersTemplate = "<<view server.bag SiteIcon width:24 height:24 spaceLink:yes label:no>> <<view server.bag spaceLink>>";
+shadows.FollowingTemplate = "<<view title SiteIcon width:24 height:24 spaceLink:yes label:no>> <<view title spaceLink>>";
 shadows.FollowTiddlersBlackList = "";
 shadows.FollowTiddlersHeading = "There are tiddlers in spaces you follow using the follow tag which use the title <<view title text>>";
 shadows.FollowTiddlersTemplate = ["* <<view server.bag SiteIcon width:24 height:24 spaceLink:yes label:no>> ",
@@ -71,12 +72,12 @@ tiddlyspace.displayServerTiddler = function(src, title, workspace, callback) {
 	var adaptor = store.getTiddlers()[0].getAdaptor();
 	var isPublic = endsWith(workspace, "_public");
 	var space = tiddlyspace.resolveSpaceName(workspace);
-	if(tiddlyspace.currentSpace.name == space) {
+	if(currentSpace == space) {
 		space = isPublic ? "public" : "private";
 	} else {
-		space = "@%0".format([space]);
+		space = "@%0".format(space);
 	}
-	var localTitle = "%0 [%1]".format([title, space]);
+	var localTitle = "%0 [%1]".format(title, space);
 	var tiddler = new Tiddler(localTitle);
 	tiddler.text = "Please wait while this tiddler is retrieved...";
 	tiddler.fields.doNotSave = "true";
@@ -108,7 +109,6 @@ tiddlyspace.displayReplyButton = function(el, tiddler) {
 		el.onclick = null;
 	} 
 	var btn = $("<button />").addClass("reply").appendTo(el);
-	var publicBag = "%0_public".format([tiddlyspace.currentSpace.name]);
 	var serverTitle = tiddler.fields["server.title"];
 	var yourTiddler = store.getTiddler(serverTitle);
 	if(yourTiddler) {
@@ -123,12 +123,12 @@ tiddlyspace.displayReplyButton = function(el, tiddler) {
 			var text = yourTiddler ? yourTiddler.text : "";
 			var newFields = {};
 			merge(newFields, config.defaultCustomFields);
-			merge(newFields, { "server.workspace": "bags/%0".format([publicBag]) });
+			merge(newFields, { "server.workspace": tiddlyspace.getCurrentWorkspace("public") });
 			var customFields = String.encodeHashMap(newFields);
 			if(customFields) {
 				story.addCustomFields(tiddlerEl, customFields);
 			}
-			var replyTemplate = "in reply to @%0:\n<<<\n%1\n<<<\n\n%2".format([tiddler.modifier, tiddler.text, text]);
+			var replyTemplate = "in reply to @%0:\n<<<\n%1\n<<<\n\n%2".format(tiddler.modifier, tiddler.text, text);
 			$("[edit=text]", tiddlerEl).val(replyTemplate);
 		} else {
 			story.displayTiddler(ev.target, serverTitle, DEFAULT_VIEW_TEMPLATE, false, null, null);
@@ -210,21 +210,16 @@ var followMacro = config.macros.followTiddlers = {
 		ev.stopPropagation();
 		return popup;
 	},
-	_getFollowerBags: function(followers) {
-		var x = [];
-		for(var i = 0; i < followers.length; i++) {
-			var name = followers[i];
-			if(name != tiddlyspace.currentSpace.name) {
-				x.push("%0_public".format([name]));
-			}
-		}
-		return x;
+	_getFollowerBags: function(followers) { // XXX: private or not?
+		return $.map(followers, function(name, i) {
+			return name != currentSpace ? "%0_public".format(name) : null;
+		});
 	},
 	getFollowers: function(callback, username) {
 		// returns a list of spaces being followed by the existing space
 		var followersCallback = function(user) {
 			if(!user.anon) {
-				scanMacro.scan(null, { searchField: "bag", searchValues: ["%0_public".format([user.name])],
+				scanMacro.scan(null, { searchField: "bag", searchValues: ["%0_public".format(user.name)],
 					spaceField: "title", template: null, tag: "follow", cache: true,
 					callback: function(tiddlers) {
 						var followers = [];
@@ -250,12 +245,10 @@ var scanMacro = config.macros.tsScan = {
 	init: function () {
 		this.scanned = {};
 	},
-	_scanCallback: function(place, jsontiddlers, options) {
-		var locale = followersMacro.locale;
-		var spaceField = options.spaceField || "bag";
+	_tiddlerfy: function(jsontiddlers, options) {
 		var tiddlers = [];
-		for(var i = 0; i < jsontiddlers.length; i++) {
-			var t = jsontiddlers[i];
+		var spaceField = options.spaceField || "bag"; // TODO: phase out use view types instead
+		$.each(jsontiddlers, function(i, t) {
 			var use = false;
 			if(!options.showBags || (options.showBags && options.showBags.contains(t.bag))) {
 				use = true;
@@ -269,7 +262,13 @@ var scanMacro = config.macros.tsScan = {
 				tiddler.fields["server.space"] = tiddlyspace.resolveSpaceName(spaceName);
 				tiddlers.push(tiddler);
 			}
-		}
+		});
+		return tiddlers;
+	},
+	_scanCallback: function(place, jsontiddlers, options) {
+		var locale = followersMacro.locale;
+		var tiddlers = scanMacro._tiddlerfy(jsontiddlers, options);
+		
 		if(options.sort) {
 			tiddlers = store.sortTiddlers(tiddlers, options.sort);
 		}
@@ -281,33 +280,38 @@ var scanMacro = config.macros.tsScan = {
 			var list = $("<ul />").appendTo(place)[0];
 			scanMacro.template(list, tiddlers, options.template);
 			if(tiddlers.length === 0) {
-				$("<li />").text(locale.noone).appendTo(list);
+				$("<li />").text(options.emptyMessage || locale.noone).appendTo(list);
+				$(list).addClass("emptyList");
 			}
 		}
 		if(options.callback) {
 			options.callback(tiddlers);
 		}
 	},
-	scan: function(place, options) {
+	constructSearchUrl: function(host, options) {
+		if(options.url) {
+			return options.url;
+		}
+		var inputs = options.searchValues;
+		var tag = options.tag;
+		var searchField = options.searchField || "title";
+		var searchQuery = [];
+		for(var i = 0; i < inputs.length; i++) {
+			searchQuery.push('%0:"%1"'.format(searchField, inputs[i]));
+		}
+		var query = searchQuery.join(" OR ");
+		query = tag ? "(%0) AND tag:%1".format(query, tag) : query;
+		query = options.query ? "%0;%1;".format(query, options.query) : query;
+		query = options.fat ? "%0&fat=y".format(query) : query;
+		return '%0/search?q=%1'.format(host, query);
+	},
+	scan: function(place, options) { // TODO: make use of list macro with url filter
 		var locale = followersMacro.locale;
 		options.template = options.template ? options.template : "ScanTemplate";
 		followMacro.getHosts(function(host, tsHost) {
-			var inputs = options.searchValues;
-			var tag = options.tag;
 			$(place).text(followersMacro.locale.pleaseWait);
 			options = options ? options: {};
-			options.filter = options.filter;
-			searchField = options.searchField || "title";
-			var searchQuery = [];
-			for(var i = 0; i < inputs.length; i++) {
-				searchQuery.push('%0:"%1"'.format([searchField, inputs[i]]));
-			}
-			var query = searchQuery.join(" OR ");
-			query = tag ? "(%0) AND tag:%1;".format([query, tag]) : query;
-			query = options.query ? "%0;%1;".format([query, options.query]) : query;
-			query = options.fat ? "%0&fat=y".format([query]) : query;
-
-			var url = '%0/search?q=%1'.format([host, query]);
+			var url = scanMacro.constructSearchUrl(host, options);
 			if(options.cache && scanMacro.scanned[url]) {
 				var tiddlers = scanMacro.scanned[url].tiddlers;
 				scanMacro._scanCallback(place, tiddlers, options);
@@ -329,7 +333,7 @@ var scanMacro = config.macros.tsScan = {
 			}
 		});
 	},
-	template: function(place, tiddlers, template) {
+	template: function(place, tiddlers, template) { // TODO: make use of list macro.
 		for(var i = 0; i < tiddlers.length; i++) {
 			var tiddler = tiddlers[i];
 			var item = $('<li class="spaceName" />').appendTo(place)[0];
@@ -340,25 +344,25 @@ var scanMacro = config.macros.tsScan = {
 	},
 	getOptions: function(paramString, tiddler) {
 		var args = paramString.parseParams("name", null, true, false, true)[0];
-		var tag = args.tag ? args.tag[0] : false;
-		var titles = args.title;
-		var spaceField = args.spaceField ? args.spaceField[0] : "bag";
-		var searchField = args.searchField ? args.searchField[0] : "title";
+		var options = { query: false, sort: false, tag: false, template: false, showBags: args.show || false,
+			hideBags: args.hide || false, filter: false, spaceField: "bag", searchField: "title", fat: false,
+			emptyMessage: false };
+		for(var name in args) {
+			if(name != "name") {
+				if(name == "fat") {
+					options[name] = true;
+				} else {
+					options[name] = args[name][0];
+				}
+			}
+		}
 		// if user has set searchField to modifier, then use the modifiers value if available otherwise use searchValues.
+		var searchField = options.searchField;
 		var searchValues = args[searchField] ? args[searchField] : args.searchValues;
 		// if neither of those were used use the first parameter
 		var defaultValues = tiddler ? [ tiddler.title ] : [];
-		searchValues = searchValues ? searchValues : ( args.name ? [args.name[0]] : defaultValues);
-		var fat = args.fat ? true : false;
-		var template = args.template ? args.template[0] : false;
-		var filter = args.filter ? args.filter[0] : false;
-		var query = args.query ? args.query[0] : false;
-		var sort = args.sort ? args.sort[0] : false;
-		var showBags = args.show ? args.show : false;
-		var hideBags = args.hide ? args.hide : false;
-		return { searchField: searchField, searchValues: searchValues,
-			template: template, filter: filter, sort: sort, hideBags: hideBags, showBags: showBags,
-			query: query, tag: tag, fat: fat, spaceField: spaceField };
+		options.searchValues = searchValues ? searchValues : ( args.name ? [args.name[0]] : defaultValues);
+		return options;
 	},
 	handler: function(place, macroName, params, wikifier, paramString, tiddler) {
 		var container = $("<div />").addClass("scanResults").appendTo(place)[0];
@@ -386,7 +390,7 @@ var followersMacro = config.macros.followers = {
 				$("<span />").text(locale.loggedOut).appendTo(container);
 			} else {
 				var options = scanMacro.getOptions(paramString);
-				$.extend(options, { searchValues: [user.name, "@%0".format([user.name])],
+				$.extend(options, { searchValues: [user.name, "@%0".format(user.name)],
 					spaceField: "bag", tag: followMacro.followTag,
 					template: options.template ? options.template : "FollowersTemplate"
 				});
@@ -394,7 +398,7 @@ var followersMacro = config.macros.followers = {
 			}
 		};
 		if(!username) {
-			followersCallback({ name: tiddlyspace.currentSpace.name });
+			followersCallback({ name: currentSpace });
 		} else {
 			followersCallback({ name: username });
 		}
@@ -421,7 +425,7 @@ var followingMacro = config.macros.following = {
 				$("<span />").text(locale.loggedOut).appendTo(container);
 			} else {
 				var options = scanMacro.getOptions(paramString);
-				$.extend(options, { searchValues: ["%0_public".format([user.name])],
+				$.extend(options, { searchValues: ["%0_public".format(user.name)],
 					tag: followMacro.followTag, searchField: "bag", spaceField: "title",
 					template: options.template ? options.template : "FollowingTemplate"
 				});
@@ -429,7 +433,7 @@ var followingMacro = config.macros.following = {
 			}
 		};
 		if(!username) {
-			followingCallback({ name: tiddlyspace.currentSpace.name });
+			followingCallback({ name: currentSpace });
 		} else {
 			followingCallback({ name: username });
 		}
@@ -456,7 +460,7 @@ config.macros.view.views.spaceLink = function(value, place, params, wikifier,
 				ev.preventDefault();
 				var el = $(ev.target);
 				tiddlyspace.displayServerTiddler(el[0], el.attr("tiddler"),
-					"bags/%0_public".format([ el.attr("tiddlyspace") ]));
+					"bags/%0_public".format( el.attr("tiddlyspace") ));
 				return false;
 			});
 		}
